#!/usr/bin/env python
"""
Runs tests in Python 2 and 3.
 
If you want tests to run before each commit you can install this as
a pre-commit hook:

   ln -sf ../../run_tests.py .git/hooks/pre-commit

The commit will now be canceled if tests fail.
"""
import os
import sys
sys.path.insert(0, '.')

def print_yellow(message):
    print('\033[0;33m{}\033[0m'.format(message))

print_yellow('Python 2')
if os.system('py.test'):
    sys.exit(1)

print_yellow('Python 3')
if os.system('py.test-3'):
    sys.exit(1)



"""
Convert a DBF file to an SQLite table.

Requires dataset: https://dataset.readthedocs.io/
"""
import dataset
from dbfread import DBF

# Change to "dataset.connect('people.sqlite')" if you want a file.
db = dataset.connect('sqlite:///:memory:')
table = db['people']

for record in DBF('files/people.dbf', lowernames=True):
    table.insert(record)

# Select and print a record just to show that it worked.
print(table.find_one(name='Alice'))



#!/usr/bin/env python2
"""
This creates the example file people.dbf.

You need the dbfpy library to run this.
"""
from __future__ import print_function
from dbfpy import dbf

def make_example_file(filename, fields, records, delete_last_record=False):
    field_names = [field[0] for field in fields]

    print('Creating', filename)
    print('  Fields:', ', '.join(field_names))
    print(' ', len(records), 'records')

    db = dbf.Dbf(filename, new=True)
    db.addField(*fields)

    for data in records:
        record = db.newRecord()
        for name, value in zip(field_names, data):
            record[name] = value
        record.store()

    if delete_last_record:
        # Delete the last one of those records.
        record.delete()
        record.store()

    try:
        db.close()
    except AttributeError:
        # This ignores the following error:
        #     self.memo.flush()
        # AttributeError: 'NoneType' object has no attribute 'flush'
        pass


make_example_file('people.dbf',
                  [('NAME', 'C', 16),
                   ('BIRTHDATE', 'D')],
                  [('Alice', (1987, 3, 1)),
                   ('Bob', (1980, 11, 12)),
                   ('Deleted Guy', (1979, 12, 22))],
                  delete_last_record=True)

make_example_file('../../testcases/memotest.dbf',
                  [('NAME', 'C', 16),
                   ('BIRTHDATE', 'D'),
                   ('MEMO', 'M')],
                  [('Alice', (1987, 3, 1), 'Alice memo'),
                   ('Bob', (1980, 11, 12), 'Bob memo'),
                   ('Deleted Guy', (1979, 12, 22), 'Deleted Guy memo')],
                  delete_last_record=True)



"""
A field parser that returns invalid values as InvalidValue objects
instead of raising ValueError.
"""
from dbfread import DBF, FieldParser, InvalidValue

class MyFieldParser(FieldParser):
    def parse(self, field, data):
        try:
            return FieldParser.parse(self, field, data)
        except ValueError:
            return InvalidValue(data)

table = DBF('files/invalid_value.dbf', parserclass=MyFieldParser)
for i, record in enumerate(table):
    for name, value in record.items():
        if isinstance(value, InvalidValue):
            print('records[{}][{!r}] == {!r}'.format(i, name, value))



"""
Add custom field parsing by subclassing FieldParser.
"""

from dbfread import DBF, FieldParser

class CustomFieldParser(FieldParser):
    def parseC(self, field, data):
        # Return strings reversed.
        return data.rstrip(b' 0').decode()[::-1]

for record in DBF('files/people.dbf', parserclass=CustomFieldParser):
    print(record['NAME'])



"""Export to CSV."""
import sys
import csv
from dbfread import DBF

table = DBF('files/people.dbf')
writer = csv.writer(sys.stdout)

writer.writerow(table.field_names)
for record in table:
    writer.writerow(list(record.values()))



"""
Return records as objects with fields as attributes.
"""
from __future__ import print_function
from dbfread import DBF

class Record(object):
    def __init__(self, items):
        for name, value in items:
            setattr(self, name, value)

for record in DBF('files/people.dbf', recfactory=Record, lowernames=True):
    print(record.name, 'was born on', record.birthdate)



"""
Return records as named tuples.

This saves a lot of memory.
"""
from collections import namedtuple
from dbfread import DBF

table = DBF('files/people.dbf', lowernames=True)

# Set record factory. This must be done after
# the table is opened because it needs the field
# names.
Record = namedtuple('Record', table.field_names)
factory = lambda lst: Record(**dict(lst))
table.recfactory = factory

for record in table:
    print(record.name)



#!/usr/bin/env python
from __future__ import print_function
import sys
from dbfread import DBF

def show(*words):
    print('  ' + ' '.join(str(word) for word in words))

def show_field(field):
    print('    {} ({} {})'.format(field.name, field.type, field.length))

def main():
    for filename in sys.argv[1:]:
        print(filename + ':')
        table = DBF(filename, ignore_missing_memofile=True)
        show('Name:', table.name)
        show('Memo File:', table.memofilename or '')
        show('DB Version:', table.dbversion)
        show('Records:', len(table))
        show('Deleted Records:', len(table.deleted))
        show('Last Updated:', table.date)
        show('Character Encoding:', table.encoding)
        show('Fields:')
        for field in table.fields:
            show_field(field)

main()



#!/usr/bin/env python
import os
import sys
import dbfread

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    sys.exit()

elif sys.argv[-1] == "test":
    os.system("./run_tests.py")
    sys.exit()

elif sys.argv[-1] == "docs":
    os.system("sphinx-build docs/ docs/_build")
    sys.exit()

setup(
    name='dbfread',
    version=dbfread.__version__,
    description='Read DBF Files with Python',
    long_description=open('README.rst', 'rt').read(),
    author=dbfread.__author__,
    author_email=dbfread.__email__,
    url=dbfread.__url__,
    package_data={'': ['LICENSE']},
    package_dir={'dbfread': 'dbfread'},
    packages = ['dbfread'],
    include_package_data=True,
    zip_safe=True,
    install_requires=[],
    license='MIT',
    classifiers=(
    ),
)



"""
Load content of a DBF file into a Pandas data frame.

The iter() is required because Pandas doesn't detect that the DBF
object is iterable.
"""
from dbfread import DBF
from pandas import DataFrame

dbf = DBF('files/people.dbf')
frame = DataFrame(iter(dbf))

print(frame)
